@@ -5,7 +5,7 @@
  *
  *    Copyright (C) 2000-2001 Stefan Petersen (spe@stacken.kth.se)
  *
- *  $Id: acconfig.h,v 1.1.1.1 2001/08/30 20:26:43 spetm Exp $
+ *  $Id: acconfig.h,v 1.6 2003/03/18 21:53:08 spetm Exp $
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,8 +22,14 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
  */
 
-/* Batch support */
-#undef NO_GUILE
+/* Support for export of PNG's */
+#undef EXPORT_PNG
+
+/* Maximum number of simultaneous loaded files */
+#undef MAX_FILES
+
+/* Default unit to display in status bar */
+#undef GERBV_DEFAULT_UNIT
 
 /* Header files */
 #undef HAVE_UNISTD_H
