@@ -2,9 +2,9 @@
  * gEDA - GNU Electronic Design Automation
  * This files is a part of gerbv.
  *
- *   Copyright (C) 2000-2001 Stefan Petersen (spe@stacken.kth.se)
+ *   Copyright (C) 2000-2002 Stefan Petersen (spe@stacken.kth.se)
  *
- * $Id: amacro.c,v 1.2 2002/01/26 11:49:57 spetm Exp $
+ * $Id: amacro.c,v 1.7 2003/05/31 13:16:09 spetm Exp $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,11 +22,12 @@
  */
 
 
-#include "amacro.h"
-#include <malloc.h>
+#include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 
+#include "amacro.h"
+
 /*
  * Allocates a new instruction structure
  */
@@ -44,7 +45,7 @@ new_instruction()
     memset(instruction, 0, sizeof(instruction_t));
     
     return instruction;
-}
+} /* new_instruction */
 
 
 /*
@@ -64,7 +65,7 @@ new_amacro()
     memset(amacro, 0, sizeof(amacro_t));
     
     return amacro;
-}
+} /* new_amacro */
 
 
 /*
@@ -106,6 +107,7 @@ parse_aperture_macro(gerb_file_t *fd)
 	    ip->next = new_instruction(); /* XXX Check return value */
 	    ip = ip->next;
 	    ip->opcode = PPUSH;
+	    amacro->nuf_push++;
 	    ip->data.ival = gerb_fgetint(fd);
 	    comma = 0;
 	    break;
@@ -191,6 +193,7 @@ parse_aperture_macro(gerb_file_t *fd)
 	    ip->next = new_instruction(); /* XXX Check return value */
 	    ip = ip->next;
 	    ip->opcode = PUSH;
+	    amacro->nuf_push++;
 	    ip->data.fval = gerb_fgetdouble(fd);
 	    if (neg) 
 		ip->data.fval = -ip->data.fval;
@@ -203,6 +206,7 @@ parse_aperture_macro(gerb_file_t *fd)
 	    return amacro;
 	default :
 	    /* Whitespace */
+	    break;
 	}
     }
 }
@@ -234,7 +238,7 @@ free_amacro(amacro_t *amacro)
     }
 	
     return;
-}
+} /* free_amacro */
 
 
 void 
@@ -274,4 +278,4 @@ print_program(amacro_t *amacro)
 	    break;
 	}
     }
-}
+} /* print_program */
