@@ -1,9 +1,9 @@
 /*
  * gEDA - GNU Electronic Design Automation
  * drill.c
- * Copyright (C) 2000-2001 Stefan Petersen (spe@stacken.kth.se)
+ * Copyright (C) 2000-2003 Andreas Andersson
  *
- * $Id: drill.c,v 1.21 2001/12/30 17:25:33 pitch Exp $
+ * $Id: drill.c,v 1.31 2003/02/08 12:15:16 pitch Exp $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,20 +43,12 @@
 #include <sys/mman.h>
 
 #include "drill.h"
-
+#include "gerb_error.h"
 
 #define NOT_IMPL(fd, s) do { \
-                             fprintf(stderr, "Not Implemented:%s\n", s); \
+                             GERB_MESSAGE("Not Implemented:%s\n", s); \
                            } while(0)
 
-#ifndef err
-#define err(errcode, a...) \
-     do { \
-           fprintf(stderr, ##a); \
-           exit(errcode);\
-     } while (0)
-#endif
-
 /* I couldn't possibly code without these */
 #undef TRUE
 #define TRUE 1
@@ -92,10 +84,11 @@ typedef struct drill_state {
     int coordinate_mode;
     double origin_x;
     double origin_y;
+    enum unit_t unit;
 } drill_state_t;
 
 /* Local function prototypes */
-static void drill_guess_format(gerb_file_t *fd, gerb_image_t *image);
+static enum unit_t drill_guess_format(gerb_file_t *fd, gerb_image_t *image);
 static int drill_parse_G_code(gerb_file_t *fd, gerb_image_t *image);
 static int drill_parse_M_code(gerb_file_t *fd, gerb_image_t *image);
 static int drill_parse_T_code(gerb_file_t *fd, drill_state_t *state, gerb_image_t *image);
@@ -110,19 +103,19 @@ parse_drillfile(gerb_file_t *fd)
     drill_state_t *state = NULL;
     gerb_image_t *image = NULL;
     gerb_net_t *curr_net = NULL;
-    char read;
+    int read;
     double x_scale = 1, y_scale = 1;
 
     state = new_state(state);
     if (state == NULL)
-	err(1, "malloc state failed\n");
+	GERB_FATAL_ERROR("malloc state failed\n");
 
     image = new_gerb_image(image);
     if (image == NULL)
-	err(1, "malloc image failed\n");
+	GERB_FATAL_ERROR("malloc image failed\n");
     curr_net = image->netlist;
 
-    drill_guess_format(fd, image);
+    state->unit = drill_guess_format(fd, image);
 
     if (image && image->format ){
 	x_scale = pow(10.0, (double)image->format->x_dec);
@@ -131,7 +124,7 @@ parse_drillfile(gerb_file_t *fd)
 
     while ((read = gerb_fgetc(fd)) != EOF) {
 
-	switch (read) {
+	switch ((char)read) {
 	case ';' :
 	    /* Comment found. Eat rest of line */
 	    eat_line(fd);
@@ -141,10 +134,10 @@ parse_drillfile(gerb_file_t *fd)
 	    eat_line(fd);
 	    break;
 	case 'G':
-	    /* G codes aren't used, for now */
+	    /* Most G codes aren't used, for now */
 	    switch(drill_parse_G_code(fd, image)) {
 	    case DRILL_G_ROUT :
-		err(1, "Rout mode data is not supported\n");
+		GERB_COMPILE_ERROR("Rout mode data is not supported\n");
 		break;
 	    case DRILL_G_DRILL :
 		break;
@@ -156,8 +149,8 @@ parse_drillfile(gerb_file_t *fd)
 		break;
 	    case DRILL_G_ZEROSET :
 		if((read = gerb_fgetc(fd)) == EOF)
-		    err(1, "Unexpected EOF\n");
-		drill_parse_coordinate(fd, read, 1.0 / x_scale, state);
+		    GERB_COMPILE_ERROR("Unexpected EOF\n");
+		drill_parse_coordinate(fd, (char)read, 1.0 / x_scale, state);
 		state->origin_x = state->curr_x;
 		state->origin_y = state->curr_y;
 		break;
@@ -196,14 +189,10 @@ parse_drillfile(gerb_file_t *fd)
 		eat_line(fd);
 	    case DRILL_M_ENDREWIND :
 		free(state);
-		/* KLUDGE. All images, regardless of input format,
-		   are returned in INCH format */
-		image->info->unit = INCH;
-
 		return image;
 		break;
 	    default:
-		err(1, "Strange M code found.\n");
+		GERB_COMPILE_ERROR("Strange M code found.\n");
 	    }
 	    break;
 
@@ -227,21 +216,37 @@ parse_drillfile(gerb_file_t *fd)
 	    curr_net->start_y = (double)state->curr_y;
 	    /* KLUDGE. This function isn't allowed to return anything
 	       but inches */
-	    if(image->info->unit == MM) {
+	    if(state->unit == MM) {
 		(double)curr_net->start_x /= 25.4;
 		(double)curr_net->start_y /= 25.4;
+		/* KLUDGE. All images, regardless of input format,
+		   are returned in INCH format */
+		curr_net->unit = INCH;
 	    }
+
 	    curr_net->stop_x = curr_net->start_x - state->origin_x;
 	    curr_net->stop_y = curr_net->start_y - state->origin_y;
 	    curr_net->aperture = state->current_tool;
 	    curr_net->aperture_state = FLASH;
 
-	    /* Find min and max of image */
-	    image->info->min_x = min(image->info->min_x, curr_net->start_x);
-	    image->info->min_y = min(image->info->min_y, curr_net->start_y);
-	    image->info->max_x = max(image->info->max_x, curr_net->start_x);
-	    image->info->max_y = max(image->info->max_y, curr_net->start_y);
-
+	    /* Find min and max of image.
+	       Mustn't forget (again) to add the hole radius */
+	    image->info->min_x =
+		min(image->info->min_x,
+		    (curr_net->start_x -
+		     image->aperture[state->current_tool]->parameter[0] / 2));
+	    image->info->min_y =
+		min(image->info->min_y,
+		    (curr_net->start_y -
+		     image->aperture[state->current_tool]->parameter[0] / 2));
+	    image->info->max_x =
+		max(image->info->max_x,
+		    (curr_net->start_x +
+		     image->aperture[state->current_tool]->parameter[0] / 2));
+	    image->info->max_y =
+		max(image->info->max_y,
+		    (curr_net->start_y +
+		     image->aperture[state->current_tool]->parameter[0] / 2));
 	    break;
 	case '%':
 	    state->curr_section = DRILL_DATA;
@@ -256,21 +261,21 @@ parse_drillfile(gerb_file_t *fd)
 		/* Unrecognised crap in the header is thrown away */
 		eat_line(fd);
 	    } else {
-		fprintf(stderr,
-			"Found unknown character %c [0x%02x], ignoring\n",
+		GERB_COMPILE_ERROR(
+			"Warning: Found ill fitting character '%c' [0x%02x] inside data, ignoring\n",
 			read, read);
 	    }
 	}
     }
 
-    fprintf(stderr, "File is missing drill End-Of-File\n");
+    GERB_COMPILE_ERROR("Warning: File is missing drill End-Of-File\n");
 
     return image;
 } /* parse_drillfile */
 
 /* Guess the format of the input file.
    Rewinds file when done */
-static void
+static enum unit_t
 drill_guess_format(gerb_file_t *fd, gerb_image_t *image)
 {
     int inch_score = 0;
@@ -278,23 +283,24 @@ drill_guess_format(gerb_file_t *fd, gerb_image_t *image)
     int length, max_length = 0;
     int leading_zeros, max_leading_zeros = 0;
     int trailing_zeros, max_trailing_zeros = 0;
-    char read;
+    int read;
     drill_state_t *state = NULL;
     int done = FALSE;
     int i;
+    enum unit_t unit;
 
     state = new_state(state);
     if (state == NULL)
-	err(1, "malloc state failed\n");
+	GERB_FATAL_ERROR("malloc state failed\n");
 
     image->format = (gerb_format_t *)malloc(sizeof(gerb_format_t));
     if (image->format == NULL) 
-	err(1, "malloc format failed\n");
+	GERB_FATAL_ERROR("malloc format failed\n");
     memset((void *)image->format, 0, sizeof(gerb_format_t));
 
     /* This is just a special case of the normal parser */
     while ((read = gerb_fgetc(fd)) != EOF && !done) {
-	switch (read) {
+	switch ((char)read) {
 	case ';' :
 	case 'F' :
 	case 'G':
@@ -328,8 +334,10 @@ drill_guess_format(gerb_file_t *fd, gerb_image_t *image)
 		   have to be rewritten sometime */
 		int local_state = 0;
 		while ((read = gerb_fgetc(fd)) != EOF &&
-		       (isdigit((int)read) || read == ',' || read == '.')) {
-		    if(read != ',' && read != '.') length ++;
+		       (isdigit(read) || read == '+' || read == '-'
+			|| read == ',' || read == '.')) {
+		    if(read != '+' && read != '-' 
+		       && read != ',' && read != '.') length ++;
 		    switch (local_state) {
 		    case 0:
 			if(read == '0') {
@@ -364,9 +372,9 @@ drill_guess_format(gerb_file_t *fd, gerb_image_t *image)
 
     /* Unfortunately, inches seem more common, so that's the default */
     if(metric_score > inch_score) {
-	image->info->unit = MM;
+	unit = MM;
     } else {
-	image->info->unit = INCH;
+	unit = INCH;
     }
 
     /* Knowing about trailing zero suppression is more important,
@@ -387,12 +395,16 @@ drill_guess_format(gerb_file_t *fd, gerb_image_t *image)
     image->format->x_dec = max_length - 2;
     image->format->y_dec = max_length - 2;
 
-    /* A bit of a kludge (or maybe wild ass guess would be more correct)
-       It seems to work, though */
-    if(image->format->omit_zeros == LEADING &&
-       image->format->x_dec <=3 && image->info->unit == INCH) {
-	++image->format->x_dec ;
-	++image->format->y_dec ;
+    /* A bit of a kludge (Or maybe wild ass guess would be more correct.
+       It seems to work, though.) It tries to cover all cases I've
+       found where the format has to be adjusted from the above
+       calculation. */
+    if( (image->format->omit_zeros == LEADING ||
+	 (max_leading_zeros == 0 && max_trailing_zeros == 0) ) &&
+	image->format->x_dec <=3 && unit == INCH) {
+
+	image->format->x_dec += 1;
+	image->format->y_dec += 1;
     }
 
     /* Restore the necessary things back to their default state */
@@ -403,7 +415,10 @@ drill_guess_format(gerb_file_t *fd, gerb_image_t *image)
 	}
     }
 
+    free(state);
     fd->ptr = 0;
+
+    return unit;
 }
 
 
@@ -414,10 +429,10 @@ drill_guess_format(gerb_file_t *fd, gerb_image_t *image)
 int
 drill_file_p(gerb_file_t *fd)
 {
-    char read[2];
+    int read;
 
-    while ((read[0] = gerb_fgetc(fd)) != EOF) {
-        switch (read[0]) {
+    while ((read = gerb_fgetc(fd)) != EOF) {
+        switch ((char)read) {
         case 'M':
             if(gerb_fgetc(fd) == '4') {
 		if(gerb_fgetc(fd) == '8') {
@@ -440,10 +455,10 @@ drill_file_p(gerb_file_t *fd)
 	       being a drill file. Stop parsing and return. */
 	    fd->ptr = 0;
 	    return 0;
-	case 10 :   /* Ignore CR/LF */
+	case 10 :  /* Ignore CR/LF */
 	case 13 :
 	    break;
-        default :
+        default : /* Eat every line that starts with something uninteresting */
             eat_line(fd);
             break;
         }
@@ -462,12 +477,26 @@ drill_parse_T_code(gerb_file_t *fd, drill_state_t *state, gerb_image_t *image)
 {
     int tool_num;
     int done = FALSE;
-    char temp;
+    int temp;
     double size;
 
+    /* Sneak a peek at what's hiding after the 'T'. Ugly fix for
+       broken headers from Orcad, which is crap */
+    temp = gerb_fgetc(fd);
+    if( !(isdigit(temp) != 0 || temp == '+' || temp =='-') ) {
+	if(temp != EOF) {
+	    GERB_COMPILE_ERROR(
+		    "Warning: Junk text found in place of tool definition.\n"
+		    "         Please ask your CAD vendor to stop doing that.\n");
+	    eat_line(fd);
+	}
+	return -1;
+    }
+    gerb_ungetc(fd);
+
     tool_num = gerb_fgetint(fd);
     if ((tool_num < TOOL_MIN) || (tool_num >= TOOL_MAX)) 
-	err(1, "Tool out of bounds: %d\n", tool_num);
+	GERB_COMPILE_ERROR("Tool out of bounds: %d\n", tool_num);
 
     /* Set the current tool to the correct one */
     state->current_tool = tool_num;
@@ -477,25 +506,34 @@ drill_parse_T_code(gerb_file_t *fd, drill_state_t *state, gerb_image_t *image)
 
     while(!done) {
 	
-	switch(temp) {
+	switch((char)temp) {
 	case 'C':
 
 	    size = read_double(fd, 1);
 
-	    if(image->info->unit == MM) {
+	    if(state->unit == MM) {
 		size /= 25.4;
+	    } else if(size >= 4.0) {
+		/* If the drill size is >= 4 inches, assume that this
+		   must be wrong and that the units are mils.
+		   The limit being 4 inches is because the smallest drill
+		   I've ever seen used is 0,3mm(about 12mil). Half of that
+		   seemed a bit too small a margin, so a third it is */
+		GERB_COMPILE_WARNING("Read a tool diameter of %g. ", size);
+		GERB_COMPILE_WARNING("Assuming this is mils and your CAD tool is broken.\n");
+		size /= 1000.0;
 	    }
 
 	    if(size <= 0 || size >= 10000) {
-		err(1, "Tool is wrong size: %g\n", size);
+		GERB_COMPILE_ERROR("Tool is wrong size: %g\n", size);
 	    } else {
 		if(image->aperture[tool_num] != NULL) {
-		    err(1, "Tool is already defined\n");
+		    GERB_COMPILE_ERROR("Tool is already defined\n");
 		} else {
 		    image->aperture[tool_num] =
 			(gerb_aperture_t *)malloc(sizeof(gerb_aperture_t));
 		    if (image->aperture[tool_num] == NULL) {
-			err(1, "malloc tool failed\n");
+			GERB_FATAL_ERROR("malloc tool failed\n");
 		    }
 		    /* There's really no way of knowing what unit the tools
 		       are defined in without sneaking a peek in the rest of
@@ -503,8 +541,8 @@ drill_parse_T_code(gerb_file_t *fd, drill_state_t *state, gerb_image_t *image)
 		    image->aperture[tool_num]->parameter[0] = size;
 		    image->aperture[tool_num]->type = CIRCLE;
 		    image->aperture[tool_num]->nuf_parameters = 1;
+		    image->aperture[tool_num]->unit = INCH;
 		}
-/*		printf("Tool %02d size %2.4g found\n", tool_num, size); */
 	    }
 	    break;
 
@@ -522,7 +560,35 @@ drill_parse_T_code(gerb_file_t *fd, drill_state_t *state, gerb_image_t *image)
 	    break;
 	}
 	if( (temp = gerb_fgetc(fd)) == EOF) {
-	    err(1, "(very) Unexpected end of file found\n");
+	    GERB_COMPILE_ERROR("(very) Unexpected end of file found\n");
+	}
+    }
+
+    /* Catch the tools that aren't defined.
+       This isn't strictly a good thing, but at least something is shown */
+    if(image->aperture[tool_num] == NULL) {
+	image->aperture[tool_num] =
+	    (gerb_aperture_t *)malloc(sizeof(gerb_aperture_t));
+	if (image->aperture[tool_num] == NULL) {
+	    GERB_FATAL_ERROR("malloc tool failed\n");
+	}
+	/* This size calculation is, of course, totally bogus */
+	image->aperture[tool_num]->parameter[0] =
+	    (double)(16 + 8 * tool_num) / 1000;
+
+	image->aperture[tool_num]->type = CIRCLE;
+	image->aperture[tool_num]->nuf_parameters = 1;
+
+	/* Oooh, this is sooo ugly. But some CAD systems seem to always
+	   use T00 at the end of the file while others that don't have
+	   tool definitions inside the file never seem to use T00 at all */
+	if(tool_num != 0) {
+	    GERB_COMPILE_ERROR(
+		    "Warning: Tool %02d used without being defined\n",
+		    tool_num);
+	    GERB_COMPILE_ERROR(
+		    "         Setting a default size of %g\"\n",
+		    image->aperture[tool_num]->parameter[0]);
 	}
     }
     
@@ -534,15 +600,16 @@ static int
 drill_parse_M_code(gerb_file_t *fd, gerb_image_t *image)
 {
     char op[3] = "  ";
+    int  read[3];
 
-    op[0] = gerb_fgetc(fd);
-    op[1] = gerb_fgetc(fd);
+    read[0] = gerb_fgetc(fd);
+    read[1] = gerb_fgetc(fd);
 
-    if ((op[0] == EOF) || (op[1] == EOF))
-	err(1, "Unexpected EOF found.\n");
-
-/*    printf("M code: %2s\n", op); */
+    if ((read[0] == EOF) || (read[1] == EOF))
+	GERB_COMPILE_WARNING("Unexpected EOF found.\n");
 
+    op[0] = read[0], op[1] = read[1], op[3] = 0;
+ 
     if (strncmp(op, "00", 2) == 0) {
 	return DRILL_M_END;
     } else if (strncmp(op, "01", 2) == 0) {
@@ -579,14 +646,15 @@ static int
 drill_parse_G_code(gerb_file_t *fd, gerb_image_t *image)
 {
     char op[3] = "  ";
+    int  read[3];
 
-    op[0] = gerb_fgetc(fd);
-    op[1] = gerb_fgetc(fd);
+    read[0] = gerb_fgetc(fd);
+    read[1] = gerb_fgetc(fd);
 
-    if ((op[0] == EOF) || (op[1] == EOF))
-	err(1, "Unexpected EOF found.\n");
+    if ((read[0] == EOF) || (read[1] == EOF))
+	GERB_COMPILE_ERROR("Unexpected EOF found.\n");
 
-/*    printf("G code: %2s\n", op); */
+    op[0] = read[0], op[1] = read[1], op[3] = 0;
 
     if (strncmp(op, "00", 2) == 0) {
 	return DRILL_G_ROUT;
@@ -622,7 +690,7 @@ drill_parse_coordinate(gerb_file_t *fd, char firstchar,
     if(state->coordinate_mode == DRILL_MODE_ABSOLUTE) {
 	if(firstchar == 'X') {
 	    state->curr_x = read_double(fd, scale_factor);
-	    if((read = gerb_fgetc(fd)) == 'Y') {
+	    if((read = (char)gerb_fgetc(fd)) == 'Y') {
 		state->curr_y = read_double(fd, scale_factor);
 	    }
 	} else {
@@ -631,7 +699,7 @@ drill_parse_coordinate(gerb_file_t *fd, char firstchar,
     } else if(state->coordinate_mode == DRILL_MODE_INCREMENTAL) {
 	if(firstchar == 'X') {
 	    state->curr_x += read_double(fd, scale_factor);
-	    if((read = gerb_fgetc(fd)) == 'Y') {
+	    if((read = (char)gerb_fgetc(fd)) == 'Y') {
 		state->curr_y += read_double(fd, scale_factor);
 	    }
 	} else {
@@ -662,11 +730,10 @@ new_state(drill_state_t *state)
 
 /* Reads one double from fd and returns it.
    If a decimal point is found, the scale factor is not used. */
-/* Too suspect. To be removed or improved. spe */
 static double
 read_double(gerb_file_t *fd, double scale_factor)
 {
-    char read;
+    int read;
     char temp[0x20];
     int i = 0;
     double result = 0;
@@ -676,9 +743,9 @@ read_double(gerb_file_t *fd, double scale_factor)
 
     read = gerb_fgetc(fd);
     while(read != EOF && i < sizeof(temp) &&
-	  (isdigit((int)read) || read == '.' || read == '+' || read == '-')) {
+	  (isdigit(read) || read == '.' || read == '+' || read == '-')) {
 	if(read == ',' || read == '.') decimal_point = TRUE;
-	temp[i++] = read;
+	temp[i++] = (char)read;
 	read = gerb_fgetc(fd);
     }
 
@@ -695,7 +762,7 @@ read_double(gerb_file_t *fd, double scale_factor)
 static void
 eat_line(gerb_file_t *fd)
 {
-    char read = gerb_fgetc(fd);
+    int read = gerb_fgetc(fd);
     
     while(read != 10 && read != 13) {
 	if (read == EOF) return;
