@@ -2,9 +2,9 @@
  * gEDA - GNU Electronic Design Automation
  * This file is a part of gerbv.
  *
- *   Copyright (C) 2000-2001 Stefan Petersen (spe@stacken.kth.se)
+ *   Copyright (C) 2000-2002 Stefan Petersen (spe@stacken.kth.se)
  *
- * $Id: gerb_file.c,v 1.8 2001/12/29 22:12:25 pitch Exp $
+ * $Id: gerb_file.c,v 1.12 2002/10/18 21:04:53 spetm Exp $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include <unistd.h>
 #endif
 #include <sys/mman.h>
+#include <errno.h>
 
 #include "gerb_file.h"
 
@@ -53,32 +54,36 @@ gerb_fopen(char *filename)
     fd->ptr = 0;
     fd->fileno = fileno(fd->fd);
     fstat(fd->fileno, &statinfo);
+    if (!S_ISREG(statinfo.st_mode)) {
+	errno = EISDIR;
+	return NULL;
+    }
+    if ((int)statinfo.st_size == 0) {
+	errno = EIO; /* More compatible with the world outside Linux */
+	return NULL;
+    }
     fd->datalen = (int)statinfo.st_size;
     fd->data = (char *)mmap(0, statinfo.st_size, PROT_READ, MAP_PRIVATE, 
 			    fd->fileno, 0);
     if(fd->data == MAP_FAILED) {
-	/* Failed to memory map file.
-	   Probable cause is that it is a directory. */
 	fclose(fd->fd);
 	free(fd);
 	fd = NULL;
     }
 
     return fd;
-}
+} /* gerb_fopen */
 
 
-char 
+int
 gerb_fgetc(gerb_file_t *fd)
 {
-    char data;
 
-    if (fd->ptr > fd->datalen)
+    if (fd->ptr > fd->datalen || fd->datalen == 0)
 	return EOF;
-    data = fd->data[fd->ptr++];
 
-    return data;
-}
+    return (int) fd->data[fd->ptr++];
+} /* gerb_fgetc */
 
 
 int
@@ -91,7 +96,7 @@ gerb_fgetint(gerb_file_t *fd)
     fd->ptr = end - fd->data;
 
     return (int)result;
-}
+} /* gerb_fgetint */
 
 
 double
@@ -104,7 +109,7 @@ gerb_fgetdouble(gerb_file_t *fd)
     fd->ptr = end - fd->data;
 
     return result;
-}
+} /* gerb_fgetdouble */
 
 
 char *
@@ -128,7 +133,7 @@ gerb_fgetstring(gerb_file_t *fd, char term)
     fd->ptr += len;
 
     return newstr;
-}
+} /* gerb_fgetstring */
 
 
 void 
@@ -138,7 +143,7 @@ gerb_ungetc(gerb_file_t *fd)
 	fd->ptr--;
 
     return;
-}
+} /* gerb_ungetc */
 
 
 void
@@ -149,4 +154,4 @@ gerb_fclose(gerb_file_t *fd)
     free(fd);
     
     return;
-}
+} /* gerb_fclose */
