@@ -2,9 +2,9 @@
  * gEDA - GNU Electronic Design Automation
  * This file is a part of gerbv.
  *
- *   Copyright (C) 2000-2001 Stefan Petersen (spe@stacken.kth.se)
+ *   Copyright (C) 2000-2002 Stefan Petersen (spe@stacken.kth.se)
  *
- * $Id: gerb_file.h,v 1.4 2001/12/29 20:16:55 spetm Exp $
+ * $Id: gerb_file.h,v 1.5 2002/10/18 21:04:53 spetm Exp $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,7 +35,7 @@ typedef struct file {
 } gerb_file_t;
 
 gerb_file_t *gerb_fopen(char *filename);
-char gerb_fgetc(gerb_file_t *fd);
+int gerb_fgetc(gerb_file_t *fd);
 int gerb_fgetint(gerb_file_t *fd);
 double gerb_fgetdouble(gerb_file_t *fd);
 char *gerb_fgetstring(gerb_file_t *fd, char term);
